@@ -1,5 +1,5 @@
 from mockups.helpers import *
 from mockups.factory import Factory
-from mockups.base import Mockup
+from mockups.base import ModelMockup
 from mockups.constraints import InvalidConstraint
 
